--[[
Name: QuestEditor
Revision: @revision@
Author(s): Angelike
Website: http://wwww.wowace.com/input_my_path_here
Documentation: www.wowace.com
SVN: svn://wowace.com/input-my-path-here
Description: Edit and saves Questsdata for LibQuestInfo
License: LGPL v2.1
]]
local AddOnName, QuestSave = ...;
local QuestEditor = {};
do -- check for missing libraries
	if not LibStub then
		print("Libstub not found, LibQuestInfo not loaded.");
		return;
	end
	local REQUIRED_LIBS = { "CallbackHandler-1.0","LibQuestInfo",};
	local missing;

	for idx, QuestEditor in ipairs(REQUIRED_LIBS) do
		if not LibStub:GetLibrary(QuestEditor, true) then
			missing = true;
			print(format("The Library %s could not be found QuestEditor not loaded.", QuestEditor));
		end
	end
	if missing then
		return;
	end
end

local LibQuestInfo = LibStub("LibQuestInfo");
local questindex, AddOnFrame = {};

-- reference global vars
local _G = _G
local table = table
local string = string
local tonumber = tonumber
local tostring = tostring

local new, del, deepDel, doAll
do -- local table stuff
	local list = setmetatable({}, {__mode='k'})
	function new(...)
		local t = next(list)
		if t then
			list[t] = nil
			for i = 1, select('#', ...) do
				t[i] = select(i, ...)
			end
			return t
		else
			return { ... }
		end
	end
	function del(t)
		setmetatable(t, nil)
		table.wipe(t)
		list[t] = true
		return nil
	end
	function deepDel(t)
		if type(t) ~= "table" then
			return nil
		end
		for k,v in pairs(t) do
			t[k] = deepDel(v)
		end
		return del(t)
	end
	function doAll(func, ...)
		for i=1, select('#', ...) do
			func(select(i, ...))
		end
	end
end

-- switch to onload
if QuestEditor.callbacks then
	QuestEditor:UnregisterAll(QuestEditor) -- unregisters all callbacks that questinfo is doing on itself (comms stuff, mostly)
	QuestEditor:UnhookDialogs()
else
	QuestEditor.callbacks = LibStub("CallbackHandler-1.0"):New(QuestEditor, nil, nil, "UnregisterAll")
end

--
-- Events (only callbacks)
--
local Updatetable
function QuestSave_EditorFrame_ScrollFrame_OnLoad(frame)
	HybridScrollFrame_CreateButtons(frame, "QuestSave_EditorFrame_ButtonTemplate");
	local width = frame:GetWidth()
	frame.update = function() QuestSave:ScrollFrame_Update() end

	for i=1, #frame.buttons do
		local button = frame.buttons[i]
		button:Reset()
		button:SetText(button:GetName())
		button:SetWidth(width)
		button:SetScript("OnClick", function(button)
			if button.HeaderKey then
				if Updatetable.Zonestate[button.HeaderKey] == 1 then
					Updatetable.Zonestate[button.HeaderKey] = 0
				else
					Updatetable.Zonestate[button.HeaderKey] = 1
				end
				QuestSave:ScrollFrame_Update(1)
				return
			end

			if button.qID then
				QuestSave_EditorFrame_LoadQuest(button.qID)
				QuestSave_EditorFrame_FrameHighlight:Show()
				QuestSave_EditorFrame_FrameHighlight:SetPoint("TOPLEFT", button, "TOPLEFT",0,0)
				QuestSave_EditorFrame_FrameHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT",0,0)
				QuestSave_EditorFrame_FrameHighlight.qid = button.qID;
			end

		end)
	end

	local buttonheigth = frame.buttons[1]:GetHeight()
	HybridScrollFrame_Update(frame, #frame.buttons * buttonheigth, #frame.buttons * buttonheigth);
end

function QuestSave:LoadQuest(qID)
	local QuestFrame = QuestSave_EditorFrame_FrameEditFrame

	-- prepare default state
	-- hide  objectives
	for i = 1, 10, 1 do
		QuestFrame["OLabel" .. i]:Hide()
		QuestFrame["OBtn" .. i]:Hide()
	end

	QuestFrame.EditMinLevel:Hide()
	QuestFrame.MinLevelLabel:SetFontObject(GameFontDisableSmallLeft)


	--print("you clicked on qid: " .. qID)
	if not QuestFrame.Questdata or QuestFrame.Questdata.Q ~= qID then
		local questdata = string.match(QuestDB[ceil(qID/100)], "|(Q"..qID..";.-|)")
		QuestFrame.Questdata = QuestEditor:UnpackQuestData(questdata)
		QuestFrame.Questdata.Q = qID
	end

	ShowUIPanel(ItemRefTooltip);
	ItemRefTooltip:SetOwner(QuestSave_EditorFrame_Frame, "ANCHOR_BOTTOM");
	ItemRefTooltip:SetHyperlink("quest:" .. qID)
	--ItemRefTooltip:Show()

	if QuestFrame.Questdata.O and QuestFrame.Questdata.O > 0 then
		for i = 1, QuestFrame.Questdata.O, 1 do
			QuestFrame["OLabel" .. i]:Show()
			QuestFrame["OBtn" .. i]:Show()
			QuestFrame["OBtn" .. i]:SetText(ObjectiveTypes[QuestFrame.Questdata["o" .. i]] or "Unknown")
		end
	end

	if QuestFrame.Questdata.T then
		QuestFrame.BtnTag:SetText(TagTypes[QuestFrame.Questdata.T] or "Unknown")
	else
		QuestFrame.BtnTag:SetText("None")
	end
	QuestFrame:Show()
end

local QuestZones = {};

function QuestSave:ScrollFrame_Update(force)
	-- refresh datatable
	if not Updatetable or Updatetable.NextUpdate < time() or force then
		if not Updatetable then
			-- first time, create defaults
			Updatetable = {
				NextUpdate = time() + 300,
				Zones = {},
				Zonestate = {},
			}
		else
			for k in pairs(Updatetable.Zones) do
				Updatetable.Zones[k] = nil
			end
		end

		Updatetable.NextUpdate = time() + 300

		for zonekey, zonevalue in pairs(QuestZones) do
			-- zone
			print(zonekey)
			print(zonevalue)
			local zonename = LibQuestInfo:GetQuestCategoryName(zonekey)

			if not Updatetable.Zonestate[zonename] then
				Updatetable.Zonestate[zonename] = 1
			else
				if Updatetable.Zonestate[zonename] ~= 1 then
					if not ZoneCache[zonename] then
						-- unserialize zonedata to display it
						ZoneCache[zonekey] = {strsplit('|', QuestZones[zonekey])};
						for k in ipairs(ZoneCache[zonekey]) do
							ZoneCache[zonekey][k] = tonumber(ZoneCache[zonekey][k])
						end
					end
					table.sort(ZoneCache[zonekey], function(a,b)
							if (QuestEditor:GetQuestName(a,1) or "ZZZ " .. a) < (QuestEditor:GetQuestName(b,1) or "ZZZ " .. b) then
								return true
							end
							return false
						end
					)
				elseif ZoneCache[zonename] and Updatetable.Zonestate[zonename] ~= 1 then
					-- clean up old zonecache to save memory
					for k in pairs(ZoneCache[zonename]) do
						ZoneCache[zonename][k] = nil;
					end
					ZoneCache[zonename] = nil;
				end
			end

			local found
			for k, zname in pairs(Updatetable.Zones) do
				if zname == zonename then
					found = true
					break
				end
			end

			if not found then
				table.insert(Updatetable.Zones, zonename)
			end
		end

		table.sort(Updatetable.Zones)
	end


	AddOnFrame.Highlight:Hide();

	local SrollFrame = AddOnFrame.ScrollFrame;
	local buttonheigth = SrollFrame.buttons[1]:GetHeight()

	local i = -HybridScrollFrame_GetOffset(SrollFrame) + 1

	for k, zonename in pairs(Updatetable.Zones) do
		local zonekey = QuestEditor:GetQuestCategoryKey(zonename)
		if i > 0 and i <= #SrollFrame.buttons then
			local button = SrollFrame.buttons[i]
			button:Reset()
			button.HeaderKey = zonename;
			button:SetButton(button)
			button:SetText(zonename)
			button.tag:SetText("#".. (QuestZones[zonekey] and QuestEditor:CountChars(QuestZones[zonekey], "|") or 0))
			if Updatetable.Zonestate[zonename] == 1 then
				button:SetNormalTexture([[Interface\Buttons\UI-PlusButton-UP]])
			end
--			QuestLogTitleButton_Resize(button)
		end
		i = i + 1
		if Updatetable.Zonestate[zonename] ~= 1 then
			for k, qid in pairs(ZoneCache[zonekey]) do
				if i > 0 and i <= #SrollFrame.buttons then
					local button = SrollFrame.buttons[i]
					button:Reset()
					button.qID = qid
					button:SetText(QuestEditor:GetQuestName(qid) or "Quest not in Cache! (QuestID: " .. qid .. ")")
					button.tag:SetText("qID: " .. qid)
					if button.qID == QuestSave_EditorFrame_FrameHighlight.qid then
						QuestSave_EditorFrame_FrameHighlight:Show()
						QuestSave_EditorFrame_FrameHighlight:SetPoint("TOPLEFT", button, "TOPLEFT",0,0)
						QuestSave_EditorFrame_FrameHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT",0,0)
					end
				end
				i = i + 1
				if i > #SrollFrame.buttons then
					break;
				end
			end
			if i > #SrollFrame.buttons then
				break;
			end
		end
	end

	for j = i, #SrollFrame.buttons do
		if j > 0 then
			SrollFrame.buttons[j]:Reset(SrollFrame.buttons[j])
			SrollFrame.buttons[j]:Disable()
		end
	end

	-- calc # lines
	local totallines = 0
	for k, zonename in pairs(Updatetable.Zones) do
		local zonekey = QuestEditor:GetQuestCategoryKey(zonename)
		totallines = totallines + 1
		if Updatetable.Zonestate[zonename] ~= 1 then
			totallines = totallines + (QuestZones[zonekey] and QuestEditor:CountChars(QuestZones[zonekey], "|") or 0)
		end
	end
	HybridScrollFrame_Update(SrollFrame, (totallines)  * buttonheigth, #SrollFrame.buttons * buttonheigth);
end

function QuestSave_EditorFrame_OnLoad(frame)
	AddOnFrame = frame; -- reference frame
	-- add scripts
	AddOnFrame:SetScript("OnShow", function() QuestSave:ScrollFrame_Update() end);
	
	AddOnFrame:SetScript("OnUpdate", function(self, elapsed)
		if not self.updatetime or self.updatetime < 0 then
			self.updatetime = 1
		end

		self.updatetime = self.updatetime - elapsed

		if self.updatetime > 0 then
			return
		end
		QuestSave:ScrollFrame_Update()
	end)
	
	QuestZones = LibQuestInfo:GetZoneKeys();
end

function QuestSave:EditorFrame_SelectFrame_OnShow(frame)

	frame.Btn1:Show()
	frame.Btn1:SetText("None")
	frame.Btn2:Show()
	frame.Btn3:Show()
	frame.Btn4:Show()
	frame.Btn5:Show()
	frame.Btn6:Show()
	frame.Btn7:Show()
	frame:SetWidth(160)
	frame:SetHeight(135)
	--frame.Zones:Hide()

	frame:ClearAllPoints()


	if frame.mode == "Tag" then
		frame:SetPoint("CENTER", frame:GetParent().BtnTag ,"CENTER",0,0)
		local i = 2
		for k, v in pairs(TagTypes) do
			frame["Btn" .. i]:SetText(v)
			frame["Btn" .. i].id = k
			i = i + 1
		end
	elseif frame.mode == "Zone" then
		frame:SetPoint("CENTER", frame:GetParent():GetParent() ,"CENTER",0,0)
		frame:SetHeight(frame:GetParent():GetParent():GetHeight())
		frame:SetWidth(frame:GetParent():GetParent():GetWidth())
	elseif frame.mode:sub(1,9) == "Objective" then
		local id = tonumber(frame.mode:sub(10))
		frame:SetID(id)
		frame:SetPoint("CENTER", frame:GetParent()["OBtn" .. id] ,"CENTER",0,0)
		local i = 2
		for k, v in pairs(ObjectiveTypes) do
			frame["Btn" .. i]:SetText(v)
			frame["Btn" .. i].id = k
			i = i + 1
		end
	end
--	print("show " .. frame:GetName())
end

function QuestSave:EditorFrame_SelectFrameButton_OnClick(button)
	local SelectFrame = _G["QuestSave_EditorFrame_FrameEditFrameSelectFrame"]
	local EditFrame = SelectFrame:GetParent()



	if SelectFrame.mode == "Tag" then
		EditFrame.Questdata.T = button.id
	elseif SelectFrame.mode == "Zone" then
		EditFrame.Questdata.Z = button.id
	elseif SelectFrame.mode:sub(1,9) == "Objective" then
		EditFrame.Questdata["o" .. button:GetID()] = button.id
	end

	--print("button clicked " .. EditFrame:GetName())
	SelectFrame:Hide()
	QuestSave_EditorFrame_LoadQuest(EditFrame.Questdata.Q)
end

function QuestSave_EditorFrame_TileButton_OnLoad(button)
	button:SetText(button:GetName())
	button.tag:Show()
	button.Reset = function(button)
		button:Enable()
		button:SetText("")
		button.tag:SetText("")
		button:SetNormalTexture("")
		button:SetHighlightTexture("")
		button:SetNormalFontObject(GameFontNormalLeft)
		button.HeaderKey = nil
		button.qID = nil
	end
	button.SetButton = function(button)
		button:SetNormalFontObject(GameFontHighlight)
		button:SetNormalTexture([[Interface\Buttons\UI-MinusButton-UP]])
		button:SetHighlightTexture([[Interface\Buttons\UI-PlusButton-Hilight]])
	end
end

function QuestEditor:ScrollButton_OnClick(button)
	-- We have a header clicked: modify, update and exit
	if button.HeaderKey == 1 then
		if QuestEditor:IsHeaderCollapsed(button.id) then
			QuestEditor:ExpandQuestHeader(button.id);
		else
			QuestEditor:CollapseQuestHeader(button.id);
		end
		QuestEditor:UpdateScrollFrame(1);
		return;
	end
	
	-- ok lets load the quest
	local QuestData = LibQuestInfo:GetQuestInfo(button.id);
	
end

-- Clean up saved Data - only for Debug

-- EOF --